/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2019                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <config.h>
#include "PerformanceTest.h"

using namespace advisor;

std::string PerformanceTest::no_comment = QObject::tr( "-- no comment --" ).toStdString();


void
PerformanceTest::adjustForTest( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->getMetric( "__time_hierarchy_ghost" );
    if ( _met == NULL )
    {
        _met = cube->defineMetric(
            "__time_hierarchy_ghost",
            "__time_hierarchy_ghost",
            "DOUBLE",
            "sec",
            "",
            TIME_METRIC_URL "#non_existent",
            "Dummy metric to execute init sequence",
            NULL,
            cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
            "1",
            cubepl_time_init_str,
            "",
            "",
            "",
            true,
            cube::CUBE_METRIC_GHOST
            );
        advisor_services->addMetric( _met, NULL );
    }


    _met = cube->getMetric( "__knl_loops_ghost" );
    if ( _met == NULL )
    {
        _met = cube->defineMetric(
            "__knl_loops_ghost",
            "__knl_loops_ghost",
            "DOUBLE",
            "sec",
            "",
            TIME_METRIC_URL "#non_existent",
            "Dummy metric to execute init sequence for knl analysis",
            NULL,
            cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
            "1",
            cubepl_knl_init_str,
            "",
            "",
            "",
            true,
            cube::CUBE_METRIC_GHOST
            );
        advisor_services->addMetric( _met, NULL );
    }

    _met = cube->getMetric( "__service_dummy_metric_mpi_wait_time" );
    if ( _met == NULL )
    {
        _met = cube->defineMetric(
            "__service_dummy_metric_mpi_wait_time",
            "__service_dummy_metric_mpi_wait_time",
            "DOUBLE",
            "sec",
            "",
            TIME_METRIC_URL "#non_existent",
            "Dummy metric to display ${without_wait_state} variable",
            NULL,
            cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
            "${without_wait_state}[${calculation::callpath::id}]",
            "",
            "",
            "",
            "",
            true,
            cube::CUBE_METRIC_GHOST
            );
        advisor_services->addMetric( _met, NULL );
    }


    add_comp_time( cube );
}

bool
PerformanceTest::scout_metrics_available( cube::CubeProxy* cube ) const
{
    if ( cube->getMetric( "mpi_latesender" ) != NULL )
    {
        return true;
    }
    if ( cube->getMetric( "mpi_latereceiver" ) != NULL )
    {
        return true;
    }
    if ( cube->getMetric( "mpi_earlyreduce" ) != NULL )
    {
        return true;
    }
    if ( cube->getMetric( "mpi_earlyscan" ) != NULL )
    {
        return true;
    }
    if ( cube->getMetric( "mpi_latebroadcast" ) != NULL )
    {
        return true;
    }
    if ( cube->getMetric( "mpi_wait_nxn" ) != NULL )
    {
        return true;
    }
    if ( cube->getMetric( "mpi_barrier_wait" ) != NULL )
    {
        return true;
    }
    if ( cube->getMetric( "mpi_finalize_wait" ) != NULL )
    {
        return true;
    }
    return false;
}




void
PerformanceTest::add_comp_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _execution     = cube->getMetric( "execution" );
    cube::Metric* _max_time      = cube->getMetric( "max_time" );
    cube::Metric* _mpi           = cube->getMetric( "mpi" );
    cube::Metric* _mpi_indicator = cube->getMetric( "mpi_indicator" );
    cube::Metric* _shmem_time    = cube->getMetric( "shmem_time" );
    cube::Metric* _omp_time      = cube->getMetric( "omp_time" );
    cube::Metric* _pthread_time  = cube->getMetric( "pthread_time" );
    cube::Metric* _opencl_time   = cube->getMetric( "opencl_time" );
    cube::Metric* _cuda_time     = cube->getMetric( "cuda_time" );
    cube::Metric* _libwrap_time  = cube->getMetric( "libwrap_time" );


    if ( _max_time == NULL )
    {
        add_max_time( cube );
    }
    if ( _execution == NULL )
    {
        add_execution_time( cube );
    }
    if ( _mpi == NULL )
    {
        add_mpi_time( cube );
    }
    if ( _mpi_indicator == NULL )
    {
        add_mpi_indicator( cube );
    }
    if ( _shmem_time == NULL )
    {
        add_shmem_time( cube );
    }
    if ( _omp_time == NULL )
    {
        add_omp_time( cube );
    }
    if ( _pthread_time == NULL )
    {
        add_pthread_time( cube );
    }
    if ( _opencl_time == NULL )
    {
        add_opencl_time( cube );
    }
    if ( _cuda_time == NULL )
    {
        add_cuda_time( cube );
    }
    if ( _libwrap_time == NULL )
    {
        add_libwrap_time( cube );
    }

    cube::Metric* _comp = cube->getMetric( "comp" );
    if ( _comp == NULL )
    {
        cube::Metric* _met = cube->defineMetric(
            "Computation",
            "comp",
            "DOUBLE",
            "sec",
            "",
            TIME_METRIC_URL "#comp",
            "Time spent on computation",
            NULL,
            cube::CUBE_METRIC_POSTDERIVED,
            "metric::execution() - metric::mpi() - metric::shmem_time() - metric::omp_time() - metric::pthread_time() - metric::opencl_time() - metric::cuda_time() - metric::libwrap_time()",
            "",
            "",
            "",
            "",
            true,
            cube::CUBE_METRIC_GHOST
            );
        advisor_services->addMetric( _met, NULL );
    }
}

void
PerformanceTest::add_max_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->defineMetric(
        "Maximal time, max_time",
        "max_time",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#execution",
        "aximal run time time, max( time )",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        "metric::time()",
        "",
        "",
        "",
        "max(arg1, arg2)",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}


void
PerformanceTest::add_execution_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->defineMetric(
        "Execution",
        "execution",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#execution",
        "Execution time (does not include time allocated for idle threads)",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        "${execution}[${calculation::callpath::id}] * ( metric::time(e) - metric::omp_idle_threads(e) )",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}

void
PerformanceTest::add_mpi_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->defineMetric(
        "MPI",
        "mpi",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#mpi",
        "Time spent in MPI calls",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        "${mpi}[${calculation::callpath::id}] * ( metric::time(e) - metric::omp_idle_threads(e))",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}


void
PerformanceTest::add_mpi_indicator( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->defineMetric(
        "MPI Indicator",
        "mpi_indicator",
        "DOUBLE",
        "",
        "",
        TIME_METRIC_URL "#mpi_indicator",
        "1 if MPI has been executed on this call path, 0 - if no",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        " ${mpi}[${calculation::callpath::id}] * ( metric::mpi(e,*) /metric::mpi(e,*) )",
        "",
        "max( arg1 , arg2)",
        "max( arg1 , arg2)",
        "max( arg1 , arg2)",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}



void
PerformanceTest::add_mpi_io_time( cube::CubeProxy* cube ) const
{
    if ( cube->getMetric( "mpi_io_individual" ) )
    {
        add_mpi_io_individual_time( cube );
    }
    if ( cube->getMetric( "mpi_io_collective" ) )
    {
        add_mpi_io_collective_time( cube );
    }

    cube::Metric* _met = cube->defineMetric(
        "MPI IO",
        "mpi_io",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#mpi_io",
        "Time spent in MPI file I/O calls",
        NULL,
        cube::CUBE_METRIC_POSTDERIVED,
        "metric::mpi_io_individual() + metric::mpi_io_collective()",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}

void
PerformanceTest::add_mpi_io_individual_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->defineMetric(
        "MPI Individual",
        "mpi_io_individual",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#mpi_io_individual",
        "Time spent in individual MPI file I/O calls",
        NULL,
        cube::CUBE_METRIC_POSTDERIVED,
        "${mpi_file_individual}[${calculation::callpath::id}] * ( metric::time(e) - metric::omp_idle_threads(e) )",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}

void
PerformanceTest::add_mpi_io_collective_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->defineMetric(
        "MPI Collective",
        "mpi_io_collective",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#mpi_io_collective",
        "Time spent in collective MPI file I/O calls",
        NULL,
        cube::CUBE_METRIC_POSTDERIVED,
        "${mpi_file_collective}[${calculation::callpath::id}] * ( metric::time(e) - metric::omp_idle_threads(e) )",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}



void
PerformanceTest::add_wait_time_mpi( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->defineMetric(
        "Waiting time in MPI",
        "wait_time_mpi",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#mpi_waiting_time",
        "Waiting time in MPI, (mpi_latesender + mpi_latereceiver + mpi_earlyreduce + mpi_earlyscan + mpi_latebroadcast + mpi_wait_nxn + mpi_barrier_wait + mpi_finalize_wait)",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        "metric::mpi_latesender() + metric::mpi_latereceiver() + metric::mpi_earlyreduce() + metric::mpi_earlyscan() + metric::mpi_latebroadcast() + metric::mpi_wait_nxn() + metric::mpi_barrier_wait() + metric::mpi_finalize_wait()",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}




void
PerformanceTest::add_shmem_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->defineMetric(
        "SHMEM",
        "shmem_time",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#shmem",
        "Time spent in SHMEM calls",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        "${shmem}[${calculation::callpath::id}] * ( metric::time(e) - metric::omp_idle_threads(e))",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}


void
PerformanceTest::add_omp_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->defineMetric(
        "OpenMP",
        "omp_time",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#omp_time",
        "Time spent in the OpenMP run-time system and API",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        "${omp}[${calculation::callpath::id}] * (metric::time(e) - metric::omp_idle_threads(e))",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}


void
PerformanceTest::add_pthread_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->defineMetric(
        "POSIX threads",
        "pthread_time",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#pthread_time",
        "Time spent in the POSIX threads API",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        "${pthread}[${calculation::callpath::id}] * (metric::time(e) - metric::omp_idle_threads(e))",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}


void
PerformanceTest::add_opencl_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _opencl_kernel_execution = cube->getMetric( "opencl_kernel_executions" );
    if ( _opencl_kernel_execution == NULL )
    {
        add_opencl_kernel_execution_time( cube );
    }

    cube::Metric* _met = cube->defineMetric(
        "OpenCL",
        "opencl_time",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#opencl_time",
        "Time spent in the OpenCL run-time system, API and on device",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        "${opencl}[${calculation::callpath::id}] * (metric::time(e) - metric::opencl_kernel_executions(e) - metric::omp_idle_threads(e))",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}

void
PerformanceTest::add_opencl_kernel_execution_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->defineMetric(
        "OpenCL kernels",
        "opencl_kernel_executions",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#opencl_kernel_executions",
        "Time spent executing OpenCL kernels",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        "${opencl_kernel_executions}[${calculation::callpath::id}] * ( metric::time(e) - metric::omp_idle_threads(e) )",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}


void
PerformanceTest::add_cuda_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _cuda_kernel_execution = cube->getMetric( "cuda_kernel_executions" );
    if ( _cuda_kernel_execution == NULL )
    {
        add_cuda_kernel_execution_time( cube );
    }

    cube::Metric* _met = cube->defineMetric(
        "CUDA",
        "cuda_time",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#cuda_time",
        "Time spent in the CUDA run-time system, API and on device",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        "${cuda}[${calculation::callpath::id}]* ( metric::time(e) - metric::cuda_kernel_executions(e) - metric::omp_idle_threads(e) )",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}


void
PerformanceTest::add_cuda_kernel_execution_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->defineMetric(
        "CUDA kernels",
        "cuda_kernel_executions",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#cuda_kernel_executions",
        "Time spent executing CUDA kernels",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        "${cuda_kernel_executions}[${calculation::callpath::id}] * ( metric::time(e) - metric::omp_idle_threads(e) )",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}



void
PerformanceTest::add_libwrap_time( cube::CubeProxy* cube ) const
{
    cube::Metric* _met = cube->defineMetric(
        "Wrapped libraries",
        "libwrap_time",
        "DOUBLE",
        "sec",
        "",
        TIME_METRIC_URL "#libwrap_time",
        "Time spent in wrapped libraries",
        NULL,
        cube::CUBE_METRIC_PREDERIVED_EXCLUSIVE,
        "${libwrap}[${calculation::callpath::id}] * ( metric::time(e) - metric::omp_idle_threads(e) )",
        "",
        "",
        "",
        "",
        true,
        cube::CUBE_METRIC_GHOST
        );
    advisor_services->addMetric( _met, NULL );
}
