/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2018                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "config.h"
#include <QtPlugin>
#include <QDebug>
#include <QPushButton>
#include <QVBoxLayout>
#include <QApplication>
#include <QStyle>
#include <QFileDialog>

#include <cstdlib>
#include <fstream>
#include <iostream>
#include <map>
#include <vector>
#include <string>

#include "Tau2Cube.h"
#include "ContextFreeServices.h"
#include "CubeProxy.h"
#include "Globals.h"

using namespace cube;
using namespace cubepluginapi;

#if QT_VERSION < 0x050000
Q_EXPORT_PLUGIN2( Tau2CubePlugin, Tau2Cube ) // ( PluginName, ClassName )
#endif

void
Tau2Cube::version( int& major, int& minor, int& bugfix ) const
{
    major  = 1;
    minor  = 0;
    bugfix = 0;
}

QString
Tau2Cube::name() const
{
    return "Tau to Cube";
}

void
Tau2Cube::opened( ContextFreeServices* service )
{
    this->service = service;

    widget = this->service->getWidget();

    QHBoxLayout* layoutOuter = new QHBoxLayout();
    widget->setLayout( layoutOuter );



    QWidget*     inner  = new QWidget();
    QVBoxLayout* layout = new QVBoxLayout( inner );
    layoutOuter->addWidget( inner );
    inner->setSizePolicy( QSizePolicy::Maximum, QSizePolicy::Maximum );

    QPushButton* selectTauProfile = new QPushButton( "Select Tau Profile" );
    connect( selectTauProfile,                     SIGNAL( clicked() ),    this, SLOT( startAction() ) );

    layout->addWidget( selectTauProfile );
}



void
Tau2Cube::startAction()
{
    QString dir = QFileDialog::getExistingDirectory( widget, tr( "Select Tau Directory" ),
                                                     tr( "" ),
                                                     QFileDialog::ShowDirsOnly
                                                     | QFileDialog::DontResolveSymlinks );
    if ( dir.compare( "" ) == 0 )
    {
        return;
    }
    try
    {
        std::vector<std::string> dirname;
        dirname.push_back( dir.toStdString() );
        CubeProxy* cube = CubeProxy::create( ALGORITHM_TAU, dirname );
        service->openCube( cube );
    }
    catch ( cube::Error e )
    {
        QString message = QString( e.what() );
        service->setMessage( message, cubepluginapi::Error );
        return;
    }
}

void
Tau2Cube::closed()
{
}

QString
Tau2Cube::getHelpText() const
{
    return "This plugin allows to open a TAU profile Directory in Cube GUI.";
}
